# Getestet mit Python 3.11.6 und PySide 6.5.3

import PySide6.QtWidgets as widgets
import PySide6.QtCore as core
import PySide6.QtGui as gui

class MeinWidget(widgets.QWidget):
    def __init__(self, parent : widgets.QWidget|None = None):
        super().__init__(parent)
        self.setCursor(core.Qt.CursorShape.BlankCursor)
        self.grabGesture(core.Qt.GestureType.PinchGesture)
        self.grabGesture(core.Qt.GestureType.TapGesture)
        self._img = gui.QImage("test.png")
        self._transformHistory : gui.QTransform = gui.QTransform()
        self._transformTotal : gui.QTransform = gui.QTransform()
        self._startPoint = core.QPoint()

    def paintEvent(self, event: gui.QPaintEvent) -> None:
        super().paintEvent(event)
        painter = gui.QPainter(self)
        painter.setTransform(self._transformTotal)
        painter.drawImage(0, 0, self._img)
        painter.end()

    def event(self, event: core.QEvent) -> bool:
        if isinstance(event, widgets.QGestureEvent):
            for g in event.gestures():
                if isinstance(g, widgets.QPinchGesture):
                    upperLeft = self.mapToGlobal(core.QPoint())
                    scale = g.totalScaleFactor()
                    angle = g.totalRotationAngle()
                    centerPoint = g.centerPoint() - upperLeft
                    if g.state() == core.Qt.GestureState.GestureStarted:
                       self._startPoint = g.centerPoint() - upperLeft
                    self._transformTotal = self._transformHistory * gui.QTransform().translate(centerPoint.x(), centerPoint.y()).rotate(angle).scale(scale, scale).translate(-self._startPoint.x(), -self._startPoint.y())
                    if g.state() == core.Qt.GestureState.GestureFinished:
                        self._transformHistory = self._transformTotal
                    self.update()
                elif isinstance(g, widgets.QTapGesture):
                    if g.state() == core.Qt.GestureState.GestureFinished:
                        print("tap", g.position())
                        self._transformTotal = gui.QTransform()
                        self._transformHistory = gui.QTransform()
                        self.update()
            return True
        else:
            return super().event(event)